import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useState, useRef, useEffect } from 'react';
import { Keyboard } from './../elements/Keyboard';
import { Label } from './../elements/Label';
import { KeyboardTypeEnumeration } from './../../shared/keyboardProps';
import { Box, Dialog, Typography, IconButton, Input } from '@mui/material';
import { dialogTitleStyle, dialogTitleContainerStyle, dialogContainerStyle, textBreakWordStyle } from './../../styles/themeExtensions';
import CloseIcon from '@mui/icons-material/Close';
import VerticalAlignBottomIcon from '@mui/icons-material/VerticalAlignBottom';
import VerticalAlignTopIcon from '@mui/icons-material/VerticalAlignTop';
import ArrowRightIcon from '@mui/icons-material/ArrowRight';
export const KeyboardDialog = (props) => {
    // States
    const [inputValue, setInputValue] = useState(props.initialInput);
    const [inputSelectionStart, setInputSelectionStart] = useState(-1);
    const [inputSelectionEnd, setInputSelectionEnd] = useState(-1);
    const [isInputFocused, setInputFocused] = useState(false);
    const [isInputValid, setInputValid] = useState(true);
    // Refs
    const inputRef = useRef(null);
    // Effects
    useEffect(() => {
        var validationResult = validate(inputValue);
        if (validationResult !== isInputValid)
            setInputValid(validationResult);
    });
    useEffect(() => {
        if (props.isVisible) {
            setInputValue(props.initialInput);
        }
        else {
        }
    }, [props.isVisible]);
    const validate = (input) => {
        if (props.onValidate)
            return props.onValidate(input);
        if (props.settings.type === KeyboardTypeEnumeration.Numeric) {
            var harmonizedInput = input.replace(',', '.');
            var numericInput = Number(harmonizedInput);
            if (isNaN(numericInput))
                return false;
            // If minimum an maximim are equal, then no validation is required
            if (props.minimumInput === props.maximumInput)
                return true;
            // Check for minimum input
            if (props.minimumInput !== undefined && props.minimumInput !== null)
                if (numericInput < props.minimumInput)
                    return false;
            // Check for maximum input
            if (props.maximumInput !== undefined && props.maximumInput !== null)
                if (numericInput > props.maximumInput)
                    return false;
        }
        return true;
    };
    const handleOnDialogKeyPress = (e) => {
        if (e.key === 'Enter') {
            props.onInputCompleted(inputValue);
            return;
        }
    };
    const handleOnDialogEntered = (node, isAppearing) => {
        if (inputRef.current)
            inputRef.current.select();
    };
    const handleInputChange = (e) => {
        // Check if the input is a number
        if (props.settings.type === KeyboardTypeEnumeration.Numeric)
            if (isNaN(Number(e.target.value)))
                return;
        // Update input state
        setInputValue(e.target.value);
        // Invoke the input method
        if (props.onInput)
            props.onInput(e.target.value);
    };
    const handleInputSelect = (e) => {
        if (!inputRef.current)
            return;
        if (isInputFocused) {
            setInputFocused(false);
            inputRef.current.setSelectionRange(inputSelectionStart, inputSelectionEnd);
            return;
        }
        var selectionStart = inputRef.current.selectionStart;
        if (selectionStart === null)
            selectionStart = inputValue.length;
        var selectionEnd = inputRef.current.selectionEnd;
        if (selectionEnd === null)
            selectionEnd = inputValue.length;
        setInputSelectionStart(selectionStart);
        setInputSelectionEnd(selectionEnd);
    };
    const handleKeyboardClick = (value, timeStamp) => {
        if (!inputRef.current)
            return;
        var newInputValue = inputValue;
        var newSelectionValue = inputSelectionStart;
        if (value === '{enter}') {
            props.onInputCompleted(inputValue);
            return;
        }
        if (value === '{bksp}') {
            if (inputSelectionStart !== inputSelectionEnd) {
                newInputValue = [inputValue.slice(0, inputSelectionStart), inputValue.slice(inputSelectionEnd)].join('');
            }
            else if (inputSelectionStart > 0) {
                newInputValue = [inputValue.slice(0, inputSelectionStart - 1), inputValue.slice(inputSelectionEnd)].join('');
                newSelectionValue--;
            }
        }
        else if (value === '{invsign}') {
            if (newInputValue.startsWith('-'))
                newInputValue = newInputValue.substring(1);
            else {
                newInputValue = `-${newInputValue}`;
                newSelectionValue++;
            }
        }
        else {
            newInputValue = [inputValue.slice(0, inputSelectionStart), value, inputValue.slice(inputSelectionEnd)].join('');
            newSelectionValue++;
        }
        setInputValue(newInputValue);
        if (props.onInput)
            props.onInput(newInputValue);
        if (newSelectionValue > newInputValue.length)
            newSelectionValue = newInputValue.length;
        setInputSelectionStart(newSelectionValue);
        setInputSelectionEnd(newSelectionValue);
        inputRef.current.focus();
        setInputFocused(true);
    };
    const renderHeader = () => {
        return (_jsx(Box, { sx: { ...dialogTitleStyle }, children: _jsxs(Box, { sx: { ...dialogTitleContainerStyle }, children: [_jsx(Typography, { variant: 'h5', sx: {
                            ...textBreakWordStyle,
                            textAlign: 'left'
                        }, children: props.title }, void 0), _jsx(Box, { sx: { minWidth: (theme) => theme.spacing(1) } }, void 0), _jsx(Box, { sx: {
                            marginLeft: 'auto'
                        }, children: _jsx(IconButton, { color: 'inherit', "aria-label": "Close", onClick: (e) => {
                                e.stopPropagation();
                                props.onClose();
                            }, children: _jsx(CloseIcon, {}, void 0) }, void 0) }, void 0)] }, void 0) }, void 0));
    };
    const renderLimits = () => {
        const showMinimumInput = props.minimumInput !== undefined && props.minimumInput !== null;
        const showMaximumInput = props.maximumInput !== undefined && props.maximumInput !== null;
        if (!showMinimumInput && !showMaximumInput)
            return undefined;
        if (props.minimumInput === props.maximumInput)
            return undefined;
        var isInputAboveMaximum = false;
        var isInputBelowMinimum = false;
        const inpuValueNumeric = Number(inputValue);
        if (!isNaN(Number(inpuValueNumeric))) {
            if (inpuValueNumeric > props.maximumInput)
                isInputAboveMaximum = true;
            if (inpuValueNumeric < props.minimumInput)
                isInputBelowMinimum = true;
        }
        return (_jsxs(Box, { sx: {
                display: 'flex',
                flexDirection: 'row',
                margin: '8px 8px 0px'
            }, children: [showMinimumInput &&
                    _jsxs(Box, { sx: {
                            flex: 'auto',
                            display: 'flex',
                            flexDirection: 'row',
                            color: isInputBelowMinimum ? (theme) => theme.palette.error.main : undefined,
                            fill: isInputBelowMinimum ? (theme) => theme.palette.error.main : undefined
                        }, children: [_jsx(VerticalAlignBottomIcon, {}, void 0), _jsx(Box, { sx: { minWidth: (theme) => theme.spacing(1) } }, void 0), _jsx(Label, { isStartAdornmentVisible: false, variant: 'body1', children: props.minimumInput }, void 0)] }, void 0), showMaximumInput &&
                    _jsxs(Box, { sx: {
                            flex: 'auto',
                            display: 'flex',
                            flexDirection: 'row',
                            justifyItems: 'flex-end',
                            justifyContent: 'flex-end',
                            color: isInputAboveMaximum ? (theme) => theme.palette.error.main : undefined,
                            fill: isInputAboveMaximum ? (theme) => theme.palette.error.main : undefined
                        }, children: [_jsx(Label, { align: 'right', isStartAdornmentVisible: false, variant: 'body1', children: props.maximumInput }, void 0), _jsx(Box, { sx: { minWidth: (theme) => theme.spacing(1) } }, void 0), _jsx(VerticalAlignTopIcon, {}, void 0)] }, void 0)] }, void 0));
    };
    const renderInputDisplay = () => {
        const showInitialInput = props.initialInput !== inputValue;
        return (_jsxs(Box, { sx: {
                overflow: 'hidden',
                display: 'flex',
                flexDirection: 'column',
                margin: (theme) => theme.spacing(1),
                border: '2px solid',
                borderRadius: '8px',
                color: isInputValid ? (theme) => theme.palette.secondary.main : (theme) => theme.palette.error.main,
                fill: isInputValid ? (theme) => theme.palette.secondary.main : (theme) => theme.palette.error.main
            }, children: [_jsxs(Box, { sx: {
                        flex: 'auto',
                        display: 'flex',
                        flexDirection: 'row',
                        alignItems: 'flex-end',
                        alignContent: 'flex-end',
                        minHeight: 32
                    }, children: [showInitialInput && _jsx(ArrowRightIcon, {}, void 0), showInitialInput &&
                            _jsx(Typography, { variant: 'body1', children: props.initialInput }, void 0)] }, void 0), _jsxs(Box, { sx: {
                        flex: 'auto',
                        display: 'flex',
                        flexDirection: 'row',
                        alignItems: 'flex-end',
                        alignContent: 'flex-end'
                    }, children: [_jsx(Input, { inputRef: inputRef, disableUnderline: true, sx: (theme) => ({
                                '&.MuiInput-root': {
                                    flex: 'auto',
                                    width: '100%',
                                    marginLeft: theme.spacing(1),
                                    border: 'none',
                                    boxSizing: 'border-box'
                                },
                                '& .Mui-focused': {
                                    outline: 'none'
                                },
                                '& .MuiInput-input': {
                                    ...theme.typography.h4,
                                    color: isInputValid ? undefined : theme.palette.error.main,
                                    fill: isInputValid ? undefined : theme.palette.error.main,
                                    caretColor: theme.palette.secondary.main
                                }
                            }), autoComplete: 'off', autoCorrect: 'off', autoCapitalize: 'off', spellCheck: false, value: inputValue, onChange: handleInputChange, onSelect: handleInputSelect }, void 0), props.displayUnit &&
                            _jsx(Typography, { variant: 'body1', sx: {
                                    marginRight: (theme) => theme.spacing(1),
                                    color: isInputValid ? undefined : (theme) => theme.palette.error.main,
                                    fill: isInputValid ? undefined : (theme) => theme.palette.error.main,
                                }, children: props.displayUnit.displayName }, void 0)] }, void 0)] }, void 0));
    };
    const renderKeyboard = () => {
        return (_jsx(Keyboard, { layout: props.settings.layout, onKeyClick: handleKeyboardClick }, void 0));
    };
    return (_jsxs(Dialog, { sx: {
            '& .MuiDialog-paper': {
                width: (props.settings.type === KeyboardTypeEnumeration.Numeric) ? 320 : undefined,
            }
        }, TransitionProps: {
            onEntered: handleOnDialogEntered
        }, onKeyPress: handleOnDialogKeyPress, fullWidth: false, maxWidth: 'md', open: props.isVisible, children: [renderHeader(), renderLimits(), renderInputDisplay(), _jsx(Box, { sx: {
                    ...dialogContainerStyle
                }, children: renderKeyboard() }, void 0)] }, void 0));
};
