import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/* eslint-disable react/react-in-jsx-scope -- Unaware of jsxImportSource */
/** @jsxImportSource @emotion/react */
import React, { useState, useEffect } from 'react';
import { LogProvider, LocalizationNamespaces } from '@brudererdev/web-extensions';
import { NotificationTypeEnumeration, NotificationAcknowledgeTypeEnumeration, TypeValidatorNET } from '@brudererdev/web-extensions';
import { KeyboardTypeEnumeration } from './../../shared/keyboardProps';
import { defaultNumericLayout, defaultAlphaNumericLayout } from './../../shared/keyboardLayouts';
import { KeyboardDialog } from './../dialogs/KeyboardDialog';
import { Box, FormControl, InputLabel, FilledInput, InputAdornment, FormHelperText } from '@mui/material';
import ArrowRightIcon from '@mui/icons-material/ArrowRight';
import InfoOutlinedIcon from '@mui/icons-material/InfoOutlined';
export const VarTextIn = (props) => {
    // Fields
    const contextName = 'VarTextIn';
    const logger = LogProvider.getLogger(contextName);
    var blurOnChangeCompleted = false;
    var isScrolling = false;
    const getFormattedValue = (value, modelVariable) => {
        const locale = (props.locale !== undefined) ? props.locale : 'de-DE';
        return TypeValidatorNET.formatValue(value, modelVariable.valueType, locale, modelVariable.valuePrecision);
    };
    // States
    const [displayValue, setDisplayValue] = useState(getFormattedValue(props.newValue ? props.newValue : props.modelVariable.value, props.modelVariable));
    const [isInputEditing, setInputEditing] = useState(false);
    const [isInputValid, setInputValid] = useState(true);
    const [isKeyboardVisible, setKeyboardVisible] = useState(false);
    const [keyboardSettings, setKeyboardSettings] = useState({ layout: defaultNumericLayout, type: KeyboardTypeEnumeration.Numeric });
    // Effects
    useEffect(() => {
        // Continuous validating
        var validationResult = validate(displayValue);
        if (validationResult !== isInputValid)
            setInputValid(validationResult);
    });
    useEffect(() => {
        setDisplayValue(getFormattedValue(props.newValue ? props.newValue : props.modelVariable.value, props.modelVariable));
    }, [props.modelVariable.modelLink.key]);
    useEffect(() => {
        if (!isInputEditing)
            setDisplayValue(getFormattedValue(props.modelVariable.value, props.modelVariable));
    }, [props.modelVariable.value]);
    useEffect(() => {
        if (props.newValue === undefined)
            setDisplayValue(getFormattedValue(props.modelVariable.value, props.modelVariable));
    }, [props.newValue]);
    const validate = (input) => {
        if (props.onValidate)
            return props.onValidate(input);
        if (TypeValidatorNET.isNumeric(props.modelVariable.valueType)) {
            // Check for numeric input
            var numericInput = Number(input);
            if (isNaN(numericInput))
                return false;
            // If minimum an maximum are equal, then no validation is required
            if (props.modelVariable.valueMinimum === props.modelVariable.valueMaximum)
                return true;
            // Check for minimum input
            if (props.modelVariable.valueMinimum !== null)
                if (numericInput < props.modelVariable.valueMinimum)
                    return false;
            // Check for maximum input
            if (props.modelVariable.valueMaximum !== null)
                if (numericInput > props.modelVariable.valueMaximum)
                    return false;
        }
        return true;
    };
    const handleInputChangeCompleted = (input) => {
        // Reset editing state
        setInputEditing(false);
        // Validate final input
        if (!validate(input)) {
            if (props.onValidationError) {
                var notification = {
                    display: {
                        link: {
                            key: 'notifications.limitvalidationerror',
                            path: 'notifications',
                            name: 'limitvalidationerror',
                            seperator: '.'
                        },
                        keyNamespace: LocalizationNamespaces.UIGeneric,
                        value: 'The value does not match the specified limits'
                    },
                    notificationType: NotificationTypeEnumeration.Fault,
                    acknowledgeType: NotificationAcknowledgeTypeEnumeration.Autohide,
                    timeStamp: Date.now()
                };
                props.onValidationError(notification);
            }
            // Reset display value
            setDisplayValue(getFormattedValue(props.newValue ? props.newValue : props.modelVariable.value, props.modelVariable));
            return;
        }
        // Clone variable props
        var varClone = { ...props.modelVariable };
        // Type checking
        if (TypeValidatorNET.isInteger(varClone.valueType)) {
            var numericValue = Number.parseInt(input);
            if (isNaN(numericValue)) {
                logger.warning(`Value of the variable '${varClone.modelLink.key}' can not be changed. New input '${input}' can not be parsed to an integer value.`);
                return;
            }
            varClone.value = numericValue;
        }
        else if (TypeValidatorNET.isFloat(varClone.valueType)) {
            var numericValue = Number.parseFloat(input);
            if ((props.modelVariable.valuePrecision !== null) && (props.modelVariable.valuePrecision < 0))
                numericValue = Number.parseFloat(numericValue.toFixed(0));
            if (isNaN(numericValue)) {
                logger.warning(`Value of the variable '${varClone.modelLink.key}' can not be changed. New input '${input}' can not be parsed to a float value.`);
                return;
            }
            varClone.value = numericValue;
        }
        else {
            varClone.value = input;
        }
        // Set final display value
        setDisplayValue(getFormattedValue(input, varClone));
        // Invoke variable changed method
        if (props.onModelVariableChanged)
            props.onModelVariableChanged(varClone);
        // Invoke text input changed method
        if (props.onTextInputChanged)
            props.onTextInputChanged(input);
    };
    const handleTextFieldTouchMove = (e) => {
        isScrolling = true;
    };
    const handleTextFieldTouchEnd = (e) => {
        if (!isScrolling && !props.isDisabled) {
            setKeyboardVisible(true);
            if (TypeValidatorNET.isString(props.modelVariable.valueType))
                setKeyboardSettings({ layout: defaultAlphaNumericLayout, type: KeyboardTypeEnumeration.AlphaNumeric });
            else
                setKeyboardSettings({ layout: defaultNumericLayout, type: KeyboardTypeEnumeration.Numeric });
        }
        isScrolling = false;
    };
    const handleTextFieldKeyPress = (e) => {
        if (e.key === 'Enter') {
            // When the input is finished, the value change handling can be completed.
            handleInputChangeCompleted(displayValue);
            // Set a mark that we are blur the element on change completed first, so that the display value is not reset. Then blur the element...
            blurOnChangeCompleted = true;
            e.target.blur();
        }
    };
    const handleTextFieldChange = (e) => {
        // Check for the correct input if the variable type is numeric
        if (TypeValidatorNET.isNumeric(props.modelVariable.valueType)) {
            if (isNaN(Number(e.target.value)))
                return;
        }
        // Set editing state
        setInputEditing(true);
        // Update input state
        setDisplayValue(e.target.value);
        // Invoke text input method
        if (props.onTextInput)
            props.onTextInput(e.target.value);
    };
    const handleTextFieldFocus = (e) => {
        e.target.select();
    };
    const handleTextFieldBlur = (e) => {
        // Reset display value on blur, but only if the change is not completed
        if (blurOnChangeCompleted) {
            blurOnChangeCompleted = false;
        }
        else {
            if (displayTextResetOnBlur) {
                // Reset editing state
                setInputEditing(false);
                // Reset display value
                setDisplayValue(getFormattedValue(props.newValue ? props.newValue : props.modelVariable.value, props.modelVariable));
                // Invoke text input method
                if (props.onTextInput)
                    props.onTextInput(props.modelVariable.value);
            }
        }
    };
    const getLabelText = () => {
        if (displayTextType === 'Label')
            return displayText;
        if (showInitialValueOnChange && (props.newValue !== null) && (props.newValue !== undefined))
            if (props.modelVariable.value !== props.newValue)
                return getFormattedValue(props.modelVariable.value, props.modelVariable);
        return '';
    };
    const getHelperText = () => {
        if (displayTextType === 'HelperText')
            return displayText;
        return '';
    };
    const renderStartAdornmentIcon = () => {
        var IconElement = props.StartAdornmentIcon;
        if (IconElement == undefined)
            IconElement = InfoOutlinedIcon;
        if (!IconElement)
            return;
        const iconSize = 24;
        const fontSize = (iconSize * 24) / 32;
        if (typeof IconElement === 'string')
            return (_jsx(Box, { component: 'span', sx: {
                    fontSize: fontSize
                }, children: IconElement }, void 0));
        return (_jsx(IconElement, { css: {
                minWidth: iconSize,
                minHeight: iconSize,
                width: iconSize,
                height: iconSize
            } }, void 0));
    };
    const renderStartAdornment = () => {
        if (!props.showStartAdornment)
            return undefined;
        return (_jsx(InputAdornment, { sx: {
                height: '100%',
                display: 'flex',
                flexDirection: 'column',
                justifyContent: 'flex-end',
                justifyItems: 'flex-end'
            }, position: "start", children: renderStartAdornmentIcon() }, void 0));
    };
    const renderEndAdornment = () => {
        return (_jsxs(InputAdornment, { position: "end", children: [_jsx(Box, { sx: {
                        height: '100%',
                        marginTop: 1,
                        fontSize: (theme) => theme.typography.body1.fontSize
                    }, children: displayUnit }, void 0), props.EndAdornment] }, void 0));
    };
    // Helpers
    const displayUnit = props.modelVariable.engineeringUnit ? props.modelVariable.engineeringUnit.displayName : "";
    const displayText = (props.onLocalize !== undefined) ? props.onLocalize(props.modelVariable.display) : props.modelVariable.display.value;
    const displayTextResetOnBlur = props.displayTextResetOnBlur !== undefined ? props.displayTextResetOnBlur : true;
    const displayTextType = props.displayTextType ? props.displayTextType : 'HelperText';
    const localizedPlaceholderText = props.localizedPlaceholderText ? props.localizedPlaceholderText : '';
    const showInitialValueOnChange = (props.showInitialValueOnChange !== undefined) ? props.showInitialValueOnChange : false;
    return (_jsxs(React.Fragment, { children: [_jsxs(FormControl, { variant: "filled", margin: 'dense', id: props.modelVariable.modelLink.key, disabled: props.isDisabled, onTouchEnd: handleTextFieldTouchEnd, onTouchMove: handleTextFieldTouchMove, onKeyPress: handleTextFieldKeyPress, sx: {
                    width: '100%',
                }, children: [_jsx(InputLabel, { color: 'secondary', disabled: props.isDisabled, children: _jsxs(Box, { sx: {
                                display: 'flex',
                                flexDirection: 'row',
                                alignContent: 'center',
                                alignItems: 'center'
                            }, children: [showInitialValueOnChange && (props.newValue !== undefined) && (props.modelVariable.value !== props.newValue) && _jsx(ArrowRightIcon, {}, void 0), getLabelText()] }, void 0) }, void 0), _jsx(FilledInput, { color: 'secondary', autoComplete: 'off', autoCorrect: 'off', autoCapitalize: 'off', disabled: props.isDisabled, type: props.type, value: displayValue, placeholder: localizedPlaceholderText, endAdornment: renderEndAdornment(), startAdornment: renderStartAdornment(), onChange: handleTextFieldChange, onFocus: handleTextFieldFocus, onBlur: handleTextFieldBlur, sx: {
                            '& .MuiInputBase-input': {
                                fontSize: '2.35rem',
                                paddingTop: '23px',
                                paddingBottom: 0,
                                caretColor: (theme) => theme.palette.secondary.main
                            }
                        } }, void 0), _jsx(FormHelperText, { sx: (theme) => ({
                            '&.MuiFormHelperText-root': {
                                opacity: 1.0,
                                overflowWrap: 'anywhere',
                                fontWeight: 'normal',
                                ...theme.typography.body1
                            }
                        }), disabled: props.isDisabled, children: getHelperText() }, void 0)] }, void 0), _jsx(KeyboardDialog, { isVisible: isKeyboardVisible, settings: keyboardSettings, title: displayText, initialInput: displayValue, minimumInput: props.modelVariable.valueMinimum, maximumInput: props.modelVariable.valueMaximum, displayUnit: props.modelVariable.engineeringUnit ? props.modelVariable.engineeringUnit : undefined, onClose: () => setKeyboardVisible(false), 
                //onInput={props.onTextInput}
                onInputCompleted: (input) => {
                    handleInputChangeCompleted(input);
                    setKeyboardVisible(false);
                } }, void 0)] }, void 0));
};
