import { jsx as _jsx } from "react/jsx-runtime";
import { useState, useEffect, useRef } from 'react';
import { createResponseTextNotification } from '@brudererdev/web-extensions';
import { GenericModelVariableRenderer } from './../renderer/GenericModelVariableRenderer';
import { getFilteredVariables } from './../../shared/variableProps';
export const ServiceVariableInjector = (props) => {
    // Fields
    const contextName = 'ServiceVariableInjector';
    // States
    const [serviceVersion, setServiceVersion] = useState(-1);
    const [changedModelVariables, setChangedModelVariables] = useState([]);
    const [componentVersion, setComponentVersion] = useState(-1);
    // Refs
    const changedModelVariablesRef = useRef(changedModelVariables);
    changedModelVariablesRef.current = changedModelVariables;
    const serviceChangeSubscriberKeyDictionaryRef = useRef({});
    // Effects
    useEffect(() => {
        // Mount
        if (props.service) {
            const registerKey = props.service.onChanges(contextName, handleServiceChanges);
            serviceChangeSubscriberKeyDictionaryRef.current[props.service.key] = registerKey;
        }
        // Unmount
        return () => {
            if (props.service) {
                const registerKey = serviceChangeSubscriberKeyDictionaryRef.current[props.service.key];
                props.service.offChanges(registerKey);
            }
            serviceChangeSubscriberKeyDictionaryRef.current = {};
        };
    }, []);
    useEffect(() => {
        if (!props.saveChangesCommand)
            return;
        if (changedModelVariables.length <= 0) {
            props.onChangesSaved(false, undefined, props.service);
            return;
        }
        if (props.onSaveChanges) {
            props.onSaveChanges(changedModelVariables).then(result => {
                if (result) {
                    setChangedModelVariables([]);
                    setComponentVersion(componentVersion + 1);
                }
                props.onChangesSaved(result, undefined, props.service);
            });
            return;
        }
        if (props.service)
            props.service.writeModelVariables(changedModelVariables, contextName)
                .then(response => {
                var result = (response.payload != undefined) ? response.payload : false;
                // Create notification
                var notification = undefined;
                if (result) {
                    setChangedModelVariables([]);
                    setComponentVersion(componentVersion + 1);
                }
                else
                    notification = createResponseTextNotification(response, props.onLocalize);
                props.onChangesSaved(result, notification, props.service);
            });
    }, [props.saveChangesCommand]);
    useEffect(() => {
        if (!props.discardChangesCommand)
            return;
        if (props.discardChangesCommand === 'All') {
            setChangedModelVariables([]);
            setComponentVersion(componentVersion + 1);
            return;
        }
        var filteredChangedModelVariables = changedModelVariablesRef.current.filter(variable => variable.modelLink.key.toLowerCase() !== props.discardChangesCommand.toLowerCase());
        setChangedModelVariables(filteredChangedModelVariables);
        setComponentVersion(componentVersion + 1);
    }, [props.discardChangesCommand]);
    useEffect(() => {
        // Invoke external changed variable list updated method
        if (props.onChangedModelVariablesUpdated)
            props.onChangedModelVariablesUpdated(changedModelVariables, props.service);
    }, [componentVersion]);
    const handleServiceChanges = (version, key) => {
        setServiceVersion(version);
    };
    const handleModelVariableChanged = (modelVariable) => {
        // Invoke external model variable changed method
        if (props.onModelVariableChanged)
            props.onModelVariableChanged(modelVariable, props.service);
        // Push the variable to the change list
        const newChangedModelVariables = [...changedModelVariables];
        var index = newChangedModelVariables.findIndex(v => v.modelLink.key === modelVariable.modelLink.key);
        if (index === -1)
            newChangedModelVariables.push(modelVariable);
        else
            newChangedModelVariables[index] = modelVariable;
        setChangedModelVariables(newChangedModelVariables);
        setComponentVersion(componentVersion + 1);
    };
    // Setup service model variables
    var modelVariables = props.service.modelVariables;
    if (props.serviceModelVariables)
        modelVariables = props.serviceModelVariables;
    // Filter variables
    var filteredModelVariables = modelVariables;
    if (props.filter)
        filteredModelVariables = getFilteredVariables(modelVariables, props.filter, props.onLocalize, props.filterFilledByStart);
    // Props assertion
    const genericModelRenderingProps = props;
    // Helpers
    const renderingType = props.renderingType ? props.renderingType : 'auto';
    return (_jsx(GenericModelVariableRenderer, { modelVariables: filteredModelVariables, modelStructureContainer: props.service.modelStructureContainer, changedModelVariables: changedModelVariables, onModelVariableChanged: handleModelVariableChanged, ...genericModelRenderingProps, renderingType: renderingType }, void 0));
};
