import { useRef, useState, useEffect } from 'react';
export var KeyPressUpdateEnumeration;
(function (KeyPressUpdateEnumeration) {
    KeyPressUpdateEnumeration[KeyPressUpdateEnumeration["none"] = 0] = "none";
    KeyPressUpdateEnumeration[KeyPressUpdateEnumeration["direct"] = 1] = "direct";
    KeyPressUpdateEnumeration[KeyPressUpdateEnumeration["memoized"] = 2] = "memoized";
})(KeyPressUpdateEnumeration || (KeyPressUpdateEnumeration = {}));
export const useKeyPress = (targetKeys, keyPressUpdate, onKeyDown, onKeyUp) => {
    // States
    const [pressedKeys, setPressedKeys] = useState([]);
    // Refs
    const internalKeysRef = useRef([]);
    const pressedKeysRef = useRef(pressedKeys);
    pressedKeysRef.current = pressedKeys;
    // Effects
    useEffect(() => {
        // Mount
        window.addEventListener('keydown', keyDownHandler);
        window.addEventListener('keyup', keyUpHandler);
        // Unmount
        return () => {
            window.removeEventListener('keydown', keyDownHandler);
            window.removeEventListener('keyup', keyUpHandler);
        };
    }, []);
    const updatePressedKeys = () => {
        var keyPressUpdateEnumeration = keyPressUpdate !== undefined ? keyPressUpdate : KeyPressUpdateEnumeration.memoized;
        switch (keyPressUpdateEnumeration) {
            case KeyPressUpdateEnumeration.none:
                return;
            case KeyPressUpdateEnumeration.direct:
                setPressedKeys(internalKeysRef.current);
                break;
            case KeyPressUpdateEnumeration.memoized: {
                if (targetKeys === undefined || targetKeys.length <= 0)
                    return;
                var areAllTargetKeysPressed = true;
                for (var key of targetKeys) {
                    var keyMap = internalKeysRef.current.find(map => map.keyboard.key === key);
                    if (!keyMap) {
                        areAllTargetKeysPressed = false;
                        break;
                    }
                }
                if (areAllTargetKeysPressed)
                    setPressedKeys(internalKeysRef.current);
                if (pressedKeysRef.current.length > 0 && !areAllTargetKeysPressed)
                    setPressedKeys([]);
                break;
            }
        }
    };
    const keyDownHandler = (e) => {
        // Return if the given key is being held down
        if (e.repeat)
            return;
        // Check for targeted keyboard keys
        if (targetKeys && targetKeys.length > 0) {
            var targetKey = targetKeys.find(key => key === e.key);
            if (!targetKey)
                return;
        }
        // Build key map
        var keyPressStartMap = { keyboard: e, startTime: Date.now() };
        // Check for existing key in array
        var existingKey = internalKeysRef.current.find(map => map.keyboard.keyCode === e.keyCode);
        if (!existingKey) {
            // Add key map
            var pressedKeysCopy = [...internalKeysRef.current];
            pressedKeysCopy.push(keyPressStartMap);
            internalKeysRef.current = pressedKeysCopy;
            updatePressedKeys();
        }
        // Invoke callback
        if (onKeyDown)
            onKeyDown(keyPressStartMap);
    };
    const keyUpHandler = (e) => {
        // Check for targeted keyboard keys
        if (targetKeys && targetKeys.length > 0) {
            var targetKey = targetKeys.find(key => key === e.key);
            if (!targetKey)
                return;
        }
        // Build key map
        var keyPressEndMap = { keyboard: e, startTime: Date.now(), endTime: Date.now(), duration: 0 };
        // Check for existing key in array
        var existingKeyIndex = internalKeysRef.current.findIndex(map => map.keyboard.keyCode === e.keyCode);
        if (existingKeyIndex >= 0) {
            // Remove key map
            var pressedKeysCopy = [...internalKeysRef.current];
            var map = pressedKeysCopy.splice(existingKeyIndex, 1);
            if (map.length > 0) {
                // Setup end key map
                keyPressEndMap.startTime = map[0].startTime;
                keyPressEndMap.duration = keyPressEndMap.endTime - keyPressEndMap.startTime;
            }
            internalKeysRef.current = pressedKeysCopy;
            updatePressedKeys();
        }
        // Invoke callback
        if (onKeyUp)
            onKeyUp(keyPressEndMap);
    };
    return [pressedKeysRef.current];
};
