import { jsx as _jsx } from "react/jsx-runtime";
import React, { useState, useRef, useEffect } from 'react';
import { LogProvider, ServiceProvider, createResponse, resolveServiceNotAvailableResponse, BrudererUserRoleEnumeration, ServiceStateEnumeration, ResponseResultEnumeration, ComponentStateEnumeration, CrossCuttingServiceKeys, resolveServiceNotRunningResponse } from '@brudererdev/web-extensions';
export const BrudererContext = React.createContext({
    activeUser: { userName: 'Anonymous', roles: [BrudererUserRoleEnumeration.Anonymous], token: '' },
    activeLanguage: 'de-DE',
    activeThemeName: 'LacyLime',
    serviceDictionary: {},
    failedServices: [],
    isStartingServices: false,
    isStoppingServices: false,
    isPLCConnected: false,
    isServiceAutoReconnectionActive: false,
    isDeveloperModeActive: false,
    registerUser: () => Promise.resolve(createResponse()),
    loginUser: () => Promise.resolve(createResponse()),
    logoutUser: () => Promise.resolve(createResponse()),
    getUsers: () => Promise.resolve(createResponse()),
    addService: (service, serviceKey) => { },
    getService: (serviceKey) => undefined,
    startServices: () => Promise.resolve(false),
    stopServices: () => Promise.resolve(false),
    changeTheme: (themeName) => { },
    setLanguage: () => { },
    localize: () => '',
    localizeRaw: () => '',
    onLocalize: () => '',
    offLocalize: () => false,
    setDeveloperMode: (enabled) => { },
    setServiceAutoReconnection: (enabled) => { },
});
// Create a service provider
const serviceProvider = new ServiceProvider();
;
export const BrudererProvider = (props) => {
    // Fields
    const contextName = 'BrudererProvider';
    const logger = LogProvider.getLogger('BrudererProvider');
    // States
    const [activeUser, setActiveUser] = useState();
    const [activeLanguage, setActiveLanguage] = useState('de-DE');
    const [activeThemeName, setActiveThemeName] = useState('LacyLime');
    const [failedServices, setFailedServices] = useState([]);
    const [isStartingServices, setStartingServices] = useState(false);
    const [isStoppingServices, setStoppingServices] = useState(false);
    const [isPLCConnected, setPLCConnected] = useState(false);
    const [isDeveloperModeActive, setDeveloperMode] = useState(false);
    const [isServiceAutoReconnectionActive, setServiceAutoReconnection] = useState(true);
    // Refs
    const serviceProviderRef = useRef(serviceProvider);
    const serviceCountRef = useRef(0);
    const serviceVersionDictionaryRef = useRef({});
    const serviceChangeSubscriberKeyDictionaryRef = useRef({});
    const localizeSubscribers = useRef({});
    const localizeSubscriptionCounter = useRef(0);
    // Effects
    useEffect(() => {
        // Mount
        bootstrapping();
        // Unmount
        return () => {
            stopServices();
        };
    }, []);
    useEffect(() => {
        checkPLCConnection().then(result => {
            setPLCConnected(result);
        });
    }, [failedServices]);
    const bootstrapping = async () => {
        setStartingServices(true);
        logger.info(`Bootstrapping Stampingpress Machine UI...`);
        const serviceKeyValuePair = Object.entries(serviceProviderRef.current.serviceDictionary).find(([key, value]) => key === CrossCuttingServiceKeys.RESTService);
        if (serviceKeyValuePair === undefined) {
            logger.error(`Bootstrapping has failed. REST service not available.`);
            return resolveServiceNotAvailableResponse('Bootstrapping', contextName, contextName, logger);
        }
        ;
        const [key, value] = serviceKeyValuePair;
        const restService = value;
        if (restService === undefined) {
            logger.error(`Bootstrapping has failed. REST service can not be resolved.`);
            return resolveServiceNotAvailableResponse('Bootstrapping', contextName, contextName, logger);
        }
        ;
        if (restService.state !== ServiceStateEnumeration.Running) {
            var response = await restService.start();
            if ((response.result !== ResponseResultEnumeration.OK) ||
                !response.payload) {
                logger.error(`Bootstrapping has failed. REST service can not be started.`);
                return resolveServiceNotRunningResponse('Bootstrapping', contextName, contextName, logger);
            }
            ;
        }
        await checkActiveUser();
        await startServices();
        setStartingServices(false);
    };
    const checkPLCConnection = async () => {
        var serviceKeyValuePair = Object.entries(serviceProviderRef.current.serviceDictionary).find(([key, value]) => key === CrossCuttingServiceKeys.ComponentService);
        if (serviceKeyValuePair === undefined)
            return false;
        var [key, value] = serviceKeyValuePair;
        var componentService = value;
        if (componentService === undefined)
            return false;
        if (componentService.state !== ServiceStateEnumeration.Running)
            return false;
        var serviceResponse = await componentService.getComponents();
        if (serviceResponse.result !== ResponseResultEnumeration.OK)
            return false;
        if (serviceResponse.payload === undefined)
            return false;
        var plcComponent = serviceResponse.payload.find((comp) => comp.name.startsWith('PLC'));
        if (plcComponent === undefined)
            return false;
        return plcComponent.state === ComponentStateEnumeration.Ok;
    };
    const checkActiveUser = async () => {
        if (activeUser)
            return;
        const anonymousLogin = {
            userName: 'Anonymous',
            password: 'Anonymous'
        };
        const developerLogin = {
            userName: 'Developer',
            password: 'Developer'
        };
        const result = await handleLoginUser(developerLogin);
    };
    const dispatchAuthenticationToken = (token) => {
        Object.entries(serviceProviderRef.current.serviceDictionary)
            .forEach(([key, value]) => {
            var hubService = value;
            if (hubService.setAuthenticationToken) {
                hubService.setAuthenticationToken(token);
                return;
            }
            ;
            var restService = value;
            if (restService.setAuthenticationToken) {
                restService.setAuthenticationToken(token);
                return;
            }
            ;
        });
    };
    const startServices = async () => {
        var result = await serviceProviderRef.current.startServices();
        var serviceCount = 0;
        var failedServices = [];
        Object.entries(serviceProviderRef.current.serviceDictionary).forEach(([key, value], index) => {
            // Register for service changes
            const registerKey = value.onChanges(contextName, handleServiceChanges);
            serviceChangeSubscriberKeyDictionaryRef.current[key] = registerKey;
            // Init service version dictionary
            serviceVersionDictionaryRef.current[key] = value.version;
            // Count services
            serviceCount++;
            // Check service state
            if (value.state !== ServiceStateEnumeration.Running)
                failedServices.push(value);
        });
        // Init service count
        serviceCountRef.current = serviceCount;
        // Set states
        setFailedServices(failedServices);
        return result;
    };
    const stopServices = async () => {
        setStoppingServices(true);
        Object.entries(serviceProviderRef.current.serviceDictionary).forEach(([key, value], index) => {
            const registerKey = serviceChangeSubscriberKeyDictionaryRef.current[key];
            value.offChanges(registerKey);
        });
        serviceChangeSubscriberKeyDictionaryRef.current = {};
        var result = await serviceProviderRef.current.stopServices();
        setStoppingServices(false);
        return result;
    };
    const handleServiceChanges = (version, key) => {
        // Update service version dictionary
        serviceVersionDictionaryRef.current[key] = version;
        // Check service state
        var failedServiceList = [...failedServices];
        var changedService = serviceProviderRef.current.serviceDictionary[key];
        var failedServiceIndex = failedServiceList.findIndex(service => service.key === key);
        if (changedService.key === CrossCuttingServiceKeys.ComponentService) {
            checkPLCConnection().then(result => {
                setPLCConnected(result);
            });
        }
        if (changedService.state !== ServiceStateEnumeration.Running) {
            if (failedServiceIndex < 0) {
                failedServiceList.push(changedService);
                setFailedServices(failedServiceList);
            }
        }
        else {
            if (failedServiceIndex >= 0) {
                failedServiceList.splice(failedServiceIndex);
                setFailedServices(failedServiceList);
            }
        }
    };
    const handleRegisterUser = async (registerCredentials) => {
        const serviceKeyValuePair = Object.entries(serviceProviderRef.current.serviceDictionary).find(([key, value]) => key === CrossCuttingServiceKeys.RESTService);
        if (serviceKeyValuePair === undefined) {
            logger.error(`Registration for user [${registerCredentials.userName}] has failed. REST service not available.`);
            return resolveServiceNotAvailableResponse('RegisterUser', contextName, contextName, logger);
        }
        ;
        const [key, value] = serviceKeyValuePair;
        const restService = value;
        if (restService === undefined) {
            logger.error(`Registration for user [${registerCredentials.userName}] has failed. REST service can not be resolved.`);
            return resolveServiceNotAvailableResponse('RegisterUser', contextName, contextName, logger);
        }
        ;
        const restResponse = await restService.post('/api/user/registeruser', registerCredentials);
        if (!restResponse.ok || !restResponse.payload) {
            logger.error(`Registration for user [${registerCredentials.userName}] has failed. Bad response [${restResponse.status}, ${restResponse.statusText}].`);
            const response = {
                result: restResponse.result,
                messageStack: restResponse.messageStack,
                payload: false
            };
            return response;
        }
        ;
        logger.info(`User [${registerCredentials.userName}] is registred and logged in.`);
        setActiveUser(restResponse.payload);
        const response = {
            result: restResponse.result,
            messageStack: restResponse.messageStack,
            payload: true
        };
        return response;
    };
    const handleLoginUser = async (loginCredentials) => {
        const serviceKeyValuePair = Object.entries(serviceProviderRef.current.serviceDictionary).find(([key, value]) => key === CrossCuttingServiceKeys.RESTService);
        if (serviceKeyValuePair === undefined) {
            logger.error(`Login for user [${loginCredentials.userName}] has failed. REST service not available.`);
            return resolveServiceNotAvailableResponse('LoginUser', contextName, contextName, logger);
        }
        ;
        const [key, value] = serviceKeyValuePair;
        const restService = value;
        if (restService === undefined) {
            logger.error(`Login for user [${loginCredentials.userName}] has failed. REST service can not be resolved.`);
            return resolveServiceNotAvailableResponse('LoginUser', contextName, contextName, logger);
        }
        ;
        const restResponse = await restService.post('/api/user/loginuser', loginCredentials);
        if (!restResponse.ok || !restResponse.payload) {
            logger.error(`Login for user [${loginCredentials.userName}] has failed. Bad response [${restResponse.status}, ${restResponse.statusText}].`);
            const response = {
                result: restResponse.result,
                messageStack: restResponse.messageStack,
                payload: false
            };
            return response;
        }
        ;
        logger.info(`User [${loginCredentials.userName}] is logged in.`);
        setActiveUser(restResponse.payload);
        dispatchAuthenticationToken(restResponse.payload.token);
        const response = {
            result: restResponse.result,
            messageStack: restResponse.messageStack,
            payload: true
        };
        return response;
    };
    const handleLogoutUser = async () => {
        const lastUserName = activeUser?.userName;
        const anonymousLogin = {
            userName: 'Anonymous',
            password: 'Anonymous'
        };
        const response = await handleLoginUser(anonymousLogin);
        if ((response.result == ResponseResultEnumeration.OK) && response.payload)
            logger.info(`User [${lastUserName}] is logged out.`);
        return response;
    };
    const handleGetUsers = async () => {
        const serviceKeyValuePair = Object.entries(serviceProviderRef.current.serviceDictionary).find(([key, value]) => key === CrossCuttingServiceKeys.RESTService);
        if (serviceKeyValuePair === undefined) {
            logger.error(`Cannot get users. REST service not available.`);
            return resolveServiceNotAvailableResponse('GetUsers', contextName, contextName, logger);
        }
        ;
        const [key, value] = serviceKeyValuePair;
        const restService = value;
        if (restService === undefined) {
            logger.error(`Cannot get users. REST service can not be resolved.`);
            return resolveServiceNotAvailableResponse('GetUsers', contextName, contextName, logger);
        }
        ;
        return await restService.get('/api/user/getusers');
    };
    const handleThemeChange = (themeName) => {
        if (props.onThemeChange)
            props.onThemeChange(themeName);
        setActiveThemeName(themeName);
    };
    const handleSetLanguage = (locale) => {
        setActiveLanguage(locale);
    };
    const handleLocalize = (content) => {
        if (Object.entries(localizeSubscribers.current).length > 0)
            return Object.entries(localizeSubscribers.current)[0][1](content);
        if (props.onLocalize)
            return props.onLocalize(content);
        if (content === null || content === undefined)
            return 'No localizable content found.';
        return content.value;
    };
    const handleLocalizeRaw = (keyNamespace, key, value, dynamicValueDictionary) => {
        var content = {
            link: {
                key: key,
                path: '',
                name: '',
                seperator: ''
            },
            keyNamespace: keyNamespace,
            value: value,
            dynamicValueDictionary
        };
        return handleLocalize(content);
    };
    const handleOnLocalize = (contextKey, callbackHandler) => {
        // Setup register key
        localizeSubscriptionCounter.current++;
        const registerKey = `${contextKey}_${localizeSubscriptionCounter.current}`;
        // Register callback
        localizeSubscribers.current[registerKey] = callbackHandler;
        logger.debug(`Component with key [${registerKey}] has subscribed on 'Localize'.`);
        logger.debug(`[${Object.entries(localizeSubscribers.current).length}] subscribers on 'Localize'.`);
        return registerKey;
    };
    const handleOffLocalize = (registerKey) => {
        // Delete callback
        var existingSubscriber = Object.entries(localizeSubscribers.current).find(([key, value]) => key === registerKey);
        if (existingSubscriber) {
            delete localizeSubscribers.current[registerKey];
            logger.debug(`Component with key [${registerKey}] has unsubscribed on 'Localize'.`);
            logger.debug(`[${Object.entries(localizeSubscribers.current).length}] subscribers on 'Localize'.`);
            return true;
        }
        else {
            logger.error(`Component with key [${registerKey}] not registered on 'Localize'.`);
            logger.debug(`[${Object.entries(localizeSubscribers.current).length}] subscribers on 'Localize'.`);
            return false;
        }
    };
    const handleSetDeveloperMode = (enabled) => {
        setDeveloperMode(enabled);
    };
    const handleSetServiceAutoReconnection = (enabled) => {
        setServiceAutoReconnection(enabled);
    };
    return (_jsx(BrudererContext.Provider, { value: {
            activeUser: activeUser,
            activeLanguage: activeLanguage,
            activeThemeName: activeThemeName,
            serviceDictionary: serviceProviderRef.current.serviceDictionary,
            failedServices: failedServices,
            isStartingServices: isStartingServices,
            isStoppingServices: isStoppingServices,
            isPLCConnected: isPLCConnected,
            isDeveloperModeActive: isDeveloperModeActive,
            isServiceAutoReconnectionActive: isServiceAutoReconnectionActive,
            registerUser: handleRegisterUser,
            loginUser: handleLoginUser,
            logoutUser: handleLogoutUser,
            getUsers: handleGetUsers,
            addService: serviceProviderRef.current.addService,
            getService: serviceProviderRef.current.getService,
            startServices: startServices,
            stopServices: stopServices,
            changeTheme: handleThemeChange,
            setLanguage: handleSetLanguage,
            localize: handleLocalize,
            localizeRaw: handleLocalizeRaw,
            onLocalize: handleOnLocalize,
            offLocalize: handleOffLocalize,
            setDeveloperMode: handleSetDeveloperMode,
            setServiceAutoReconnection: handleSetServiceAutoReconnection,
        }, children: props.children }, void 0));
};
